# coding=utf-8

import unittest

from url_robots.utils import clean_pattern, join_patterns, create_rules


class CleanPatternTestCase(unittest.TestCase):

    def clean_equal(self, pattern, result):
        self.assertEqual(clean_pattern(pattern), '/' + result + '*')

    # special sequences tests
    def test_match_contests_of_group(self):
        self.clean_equal(r'ab(123)c\1d', 'ab*c*d')

    def test_start_n_end_of_string(self):
        self.clean_equal(r'\Aabc', 'abc')
        self.clean_equal(r'\Zabc', 'abc')

    def test_empty_string_at_bound_of_word(self):
        self.clean_equal(r'\babc', 'abc')
        self.clean_equal(r'\Babc', 'abc')

    def test_digits(self):
        self.clean_equal(r'\dabc', '*abc')
        self.clean_equal(r'\Dabc', '*abc')

    def test_spaces(self):
        self.clean_equal(r'\sabc', '*abc')
        self.clean_equal(r'\Sabc', '*abc')

    def test_letters(self):
        self.clean_equal(r'\wabc', '*abc')
        self.clean_equal(r'\Wabc', '*abc')

    def test_backslash(self):
        self.clean_equal(r'\\abc', r'%5Cabc')

    # test special characters
    def test_any_character(self):
        self.clean_equal(r'a.b', 'a*b')

    def test_begin_of_string(self):
        self.clean_equal(r'^ab', 'ab')

    def test_end_of_string(self):
        self.assertEqual(clean_pattern(r'ab$'), u'/ab$')
        self.assertEqual(clean_pattern(r'ab.$'), u'/ab*')

    def test_greedy_repetitions(self):
        self.clean_equal(r'a.*b', 'a*b')
        self.clean_equal(r'a.+b', 'a*b')
        self.clean_equal(r'a.?b', 'a*b')
        self.clean_equal(r'a.{2,3}b', 'a*b')

    def test_non_greedy_repetitions(self):
        self.clean_equal(r'a.*?b', 'a*b')
        self.clean_equal(r'a.+?b', 'a*b')
        self.clean_equal(r'a.??b', 'a*b')
        self.clean_equal(r'a.{2,3}?b', 'a*b')

    def test_set(self):
        self.clean_equal(r'a[123]b', 'a*b')

    def test_or(self):
        self.clean_equal(r'a(2|3)b', 'a*b')

    def test_subpattern(self):
        self.clean_equal(r'a(123)b', 'a*b')

    def test_flags(self):
        self.clean_equal(r'(?sux)ab', 'ab')

    def test_non_grouping(self):
        self.clean_equal(r'a(?:23)b', 'a*b')

    def test_grouping(self):
        self.clean_equal(r'a(?P<name>23)b', 'a*b')

    def test_group_matching(self):
        self.clean_equal(r'a(?P<name>23)b', 'a*b')

        self.clean_equal(r'a(?=23)b', 'a*b')
        self.clean_equal(r'a(?!23)b', 'a*b')

        self.clean_equal(r'a(?<=23)b', 'a*b')
        self.clean_equal(r'a(?<!23)b', 'a*b')

        self.clean_equal(r'(?P<name>)a(?(name)2|3)b', '*a*b')

    def test_comments(self):
        self.clean_equal(r'a(?#comments)b', 'ab')

    def test_urlquote(self):
        # escaped star and dollar
        self.clean_equal(r'star-\*', 'star-%2A')
        self.clean_equal(r'dollar-\$', 'dollar-%24')
        # some russian language
        self.clean_equal(u'path/в никуда/', r'path/%D0%B2%20%D0%BD%D0%B8%D0%BA%D1%83%D0%B4%D0%B0/')


class JoinPatternTestCase(unittest.TestCase):
    def join_equal(self, pattern1, pattern2, pattern1_2):
        self.assertEqual(join_patterns(pattern1, pattern2), pattern1_2)

    # concatenation of ^2, 2$, ^2$, 2, '' tests
    def test_caret_1(self):
        self.join_equal('^1', '^2', '^12')
        self.join_equal('^1', '2$', '^1.2$')
        self.join_equal('^1', '^2$', '^12$')
        self.join_equal('^1', '2', '^1.2')
        self.join_equal('^1', '', '^1')

    def test_1_dollar(self):
        self.join_equal('1$', '^2', '1$')
        self.join_equal('1$', '2$', '1$')
        self.join_equal('1$', '^2$', '1$')
        self.join_equal('1$', '2', '1$')
        self.join_equal('1$', '', '1$')

    def test_caret_1_dollar(self):
        self.join_equal('^1$', '^2', '^1$')
        self.join_equal('^1$', '2$', '^1$')
        self.join_equal('^1$', '^2$', '^1$')
        self.join_equal('^1$', '2', '^1$')
        self.join_equal('^1$', '', '^1$')

    def test_1(self):
        self.join_equal('1', '^2', '12')
        self.join_equal('1', '2$', '1.2$')
        self.join_equal('1', '^2$', '12$')
        self.join_equal('1', '2', '1.2')
        self.join_equal('1', '', '1')


class CreateRulesTestCase(unittest.TestCase):
    def setUp(self):
        self.expected_rules = '''Allow: /profiles$
Disallow: /profile/*/private*
Allow: /profile/*/public*'''

    def test_create_rules_for_profiles(self):
        rules = create_rules('url_robots.tests.urls')
        self.assertEqual(rules, self.expected_rules)




from django.shortcuts import render

from django2_url_robots.utils import create_rules


def robots_txt(request, template='robots.txt'):
    rules = create_rules()
    content_type = 'text/plain; charset=utf-8'

    return render(request, template, {'rules': rules}, content_type=content_type)



import sre_parse
from django import get_version
from distutils.version import StrictVersion
from sre_constants import LITERAL, AT, AT_END

_NEWEST_DJANGO = StrictVersion(get_version()) >= StrictVersion('2.0')

try:
    # Python 3
    from urllib.parse import quote, unquote

    unichr = chr
except ImportError:
    # Python 2
    from urllib import quote, unquote

try:
    from django.core.urlresolvers import get_urlconf, get_resolver, RegexURLResolver
    from django.conf.urls import url
except ImportError:
    from django.urls import get_urlconf, get_resolver, URLResolver
    from django.urls import path, re_path


def robots_decorator(url_function):
    """
    Decorator for django.conf.urls.url
    """

    def url_extended(regex, view, kwargs=None, name=None, robots_allow=None):
        resolver_or_pattern = url_function(regex, view, kwargs=kwargs, name=name)

        resolver_or_pattern.robots_allow = robots_allow
        return resolver_or_pattern

    return url_extended


if _NEWEST_DJANGO is True:
    path = robots_decorator(path)
    re_path = robots_decorator(re_path)
else:
    url = robots_decorator(url)


def create_rules(urlconf=None):
    """
    Creates rules from conf
    """
    if urlconf is None:
        urlconf = get_urlconf()

    root_resolver = get_resolver(urlconf)
    rule_list = create_rule_list(root_resolver, '')

    return u'\n'.join(rule_list)


def create_rule_list(parent_resolver, abs_pattern):
    """
    Creates usable rule list
    """
    rule_list = []

    for resolver in parent_resolver.url_patterns:
        try:
            pattern = join_patterns(abs_pattern, resolver.regex.pattern)
        except AttributeError:
            pattern = join_patterns(abs_pattern, resolver.pattern)
        rule = ''
        robots_allow = getattr(resolver, 'robots_allow', None)

        if robots_allow is None:
            pass
        elif robots_allow:
            rule = 'Allow: '
        else:
            rule = 'Disallow: '

        if rule:
            path = clean_pattern(pattern)
            rule += path
            rule_list.append(rule)

        try:
            if isinstance(resolver, RegexURLResolver):
                rule_list += create_rule_list(resolver, pattern)
        except:
            if isinstance(resolver, URLResolver):
                rule_list += create_rule_list(resolver, pattern)

    return rule_list


def join_patterns(pattern1, pattern2):
    """
    Joins URL patterns
    """
    if pattern1.endswith('$'):
        return pattern1
    try:
        if pattern2.startswith('^'):
            return pattern1 + pattern2[1:]
        if pattern2:
            return pattern1 + '.' + pattern2
    except AttributeError:
        if str(pattern2).startswith('^'):
            return pattern1 + str(pattern2)[1:]
        if pattern2:
            return pattern1 + '.' + str(pattern2)

    return pattern1


def clean_pattern(pattern):
    """
    Cleans URL patterns
     * pattern => token
     * '2'     => ('literal', 50)
     * '2|3'   => ('in', [('literal', 50), ('literal', 51)])
    """
    star = '*'
    parsed = sre_parse.parse(pattern)
    literals = []

    for token in parsed:
        if token[0] == LITERAL:
            character = quote(unichr(token[1]).encode('utf8'))
            literals.append(character)
        elif token[0] == AT:
            pass

        elif literals[-1:] != [star]:
            literals.append(star)

    rule = '/' + ''.join(literals)

    if parsed and not rule.endswith(star):
        if parsed[-1] == (AT, AT_END):
            rule += '$'
        else:
            rule += star

    return rule


#coding=utf-8


from url_robots.utils import url

urlpatterns = [
    url(r'^s$', lambda x: (), name='profiles', robots_allow=True),
    url(r'^/(?P<nick>\w+)$', lambda x: ()),
    url(r'^/(?P<nick>\w+)/private', lambda x: (), name='profile_private', robots_allow=False),
    url(r'^/(?P<nick>\w+)/public', lambda x: (), name='profile_public', robots_allow=True),
    ]


from setuptools import setup

long_description = open('README.rst').read()

setup(
    name='django2-url-robots',
    version='3.2.2',
    description='Django robots.txt generator',
    long_description=long_description,
    url='https://github.com/maximekl/django2-url-robots',
    author='Maxime Klampas',
    author_email='mklampas@gmail.com',
    license='Python Software Foundation License',
    packages=['django2_url_robots', 'django2_url_robots.tests'],
    package_data={'django2_url_robots': ['templates/*.*']},
    platforms=["any"],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Python Software Foundation License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Environment :: Web Environment',
        'Framework :: Django',
        ],
)
